package org.netkernel.client.ftp.endpoint;

import java.io.OutputStream;
import java.net.URI;
import java.net.URLConnection;

import org.netkernel.client.ftp.representation.FTPConnectionRepresentation;
import org.netkernel.client.ftp.representation.RemoteFTPRepresentation;
import org.netkernel.client.ftp.util.FTPUtil;
import org.netkernel.layer0.nkf.INKFRequest;
import org.netkernel.layer0.nkf.INKFRequestContext;
import org.netkernel.layer0.nkf.INKFResponse;
import org.netkernel.layer0.representation.IBinaryStreamRepresentation;
import org.netkernel.layer0.urii.ValueSpace;
import org.netkernel.module.standard.endpoint.StandardAccessorImpl;

import com.enterprisedt.net.ftp.FileTransferClient;

public class FTPSchemeAccessor extends StandardAccessorImpl
{
	public void onSource(INKFRequestContext aContext) throws Exception
	{
		URI uri = URI.create(aContext.getThisRequest().getIdentifier());
		FTPConnectionRepresentation connection=getConnection(aContext,uri);

		String path=FTPUtil.getPath(uri);
		RemoteFTPRepresentation rep=new RemoteFTPRepresentation(path,  connection, 1024);
		INKFResponse resp=aContext.createResponseFrom(rep);
		String contentType=URLConnection.getFileNameMap().getContentTypeFor(path);
		resp.setMimeType(contentType);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}
	
	public void onSink(INKFRequestContext aContext) throws Exception
	{
		IBinaryStreamRepresentation primary=aContext.sourcePrimary(IBinaryStreamRepresentation.class);
		
		URI uri = URI.create(aContext.getThisRequest().getIdentifier());
		FTPConnectionRepresentation connection=getConnection(aContext,uri);
		
		String path=FTPUtil.getPath(uri);
		synchronized(connection)
		{
			FileTransferClient client=connection.getClient();
			String filename=FTPUtil.changeDirAndGetFilename(connection, path);
			OutputStream os=client.uploadStream(filename);
			primary.write(os);
			os.flush();
			os.close();
		}
	}
	
	public void onExists(INKFRequestContext aContext) throws Exception
	{
		URI uri = URI.create(aContext.getThisRequest().getIdentifier());
		FTPConnectionRepresentation connection=getConnection(aContext,uri);

		String path=FTPUtil.getPath(uri);
		boolean result;
		synchronized(connection)
		{	String filename=FTPUtil.changeDirAndGetFilename(connection, path);
			FileTransferClient client=connection.getClient();
			result = client.exists(filename);
		}
		INKFResponse resp=aContext.createResponseFrom(result);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}
	
	public void onDelete(INKFRequestContext aContext) throws Exception
	{
		URI uri = URI.create(aContext.getThisRequest().getIdentifier());
		FTPConnectionRepresentation connection=getConnection(aContext,uri);

		String path=FTPUtil.getPath(uri);
		synchronized(connection)
		{	String filename=FTPUtil.changeDirAndGetFilename(connection, path);
			FileTransferClient client=connection.getClient();
			client.deleteFile(filename);
		}
		INKFResponse resp=aContext.createResponseFrom(true);
		resp.setExpiry(INKFResponse.EXPIRY_ALWAYS);
	}
	
	private FTPConnectionRepresentation getConnection(INKFRequestContext aContext, URI aIdentifier) throws Exception
	{	String connectionURI = FTPUtil.toConnectionURIString(aIdentifier);
		INKFRequest req=aContext.createRequest("pbv:literal");
		ValueSpace s=new ValueSpace(1);
		s.put("pbv:literal", connectionURI, null);
		req.injectRequestScope(s);
		req.setRepresentationClass(FTPConnectionRepresentation.class);
		return (FTPConnectionRepresentation)aContext.issueRequest(req);
	}
}
