package org.netkernel.client.ftp.representation;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.util.Iterator;

import org.netkernel.client.ftp.util.Credentials;
import org.netkernel.client.ftp.util.FTPUtil;
import org.netkernel.client.ftp.util.PathAndFile;
import org.netkernel.layer0.nkf.INKFExpiryFunction;
import org.netkernel.layer0.nkf.NKFException;
import org.netkernel.layer0.representation.IReadableBinaryStreamRepresentation;
import org.netkernel.layer0.util.Utils;
import org.netkernel.urii.impl.NetKernelException;

import com.enterprisedt.net.ftp.FTPException;
import com.enterprisedt.net.ftp.FTPTransferType;
import com.enterprisedt.net.ftp.FileTransferClient;

public class RemoteFTPRepresentation implements IReadableBinaryStreamRepresentation, INKFExpiryFunction
{
	private final FTPConnectionRepresentation mConnection;
	private final String mPath;
	private final long mOriginalLastModified;
	private final long mLength;
	private final byte[] mBytes;
	
	public RemoteFTPRepresentation(String aPath, FTPConnectionRepresentation aConnection, int aLengthThreshold) throws IOException, FTPException, NKFException
	{	
		mConnection=aConnection;
		mPath=aPath;
		
		String filename;
		FileTransferClient client;
		synchronized(mConnection)
		{	try
			{	filename=FTPUtil.changeDirAndGetFilename(mConnection,mPath);
				client=mConnection.getClient();
				if (!client.exists(filename))
				{	throw new NKFException("Remote FTP File Not Found",aPath);
				}
			}
			catch (FTPException e)
			{	throw new NKFException("Remote FTP File Not Found",aPath);
			}
			
			mLength=client.getSize(filename);
			mOriginalLastModified=client.getModifiedTime(filename).getTime();
			
			if (mLength<aLengthThreshold)
			{	mBytes=client.downloadByteArray(filename);
			}
			else
			{	mBytes=null;
			}
		}
		//System.out.println("remotedir: "+client.getRemoteDirectory());
		//System.out.println("len: "+mLength);
	}
	
	
	public int getContentLength()
	{	return (int)mLength;
	}

	public InputStream getInputStream() throws IOException
	{	
		if (mBytes!=null)
		{	return new ByteArrayInputStream(mBytes);
		}
		else
		{	try
			{	synchronized(mConnection)
				{	String filename=FTPUtil.changeDirAndGetFilename(mConnection,mPath);
					FileTransferClient client=mConnection.getClient();
					return client.downloadStream(filename);
				}
			}
			catch (FTPException e)
			{	throw new IOException(e.toString());
			}
		}
			
	}

	public String getEncoding()
	{	return null;
	}

	public void write(OutputStream aStream) throws IOException
	{	InputStream is=getInputStream();
		Utils.pipe(is, aStream);
	}
	
	public boolean isExpired(long aNow)
	{
		// TODO Auto-generated method stub
		return true;
	}

}
