package org.netkernel.client.ftp.util;

import com.enterprisedt.net.ftp.FTPException;
import com.enterprisedt.net.ftp.FTPFile;
import com.enterprisedt.net.ftp.FTPTransferType;
import com.enterprisedt.net.ftp.FileTransferClient;
import org.netkernel.client.ftp.representation.FTPConnectionRepresentation;
import org.netkernel.urii.impl.NetKernelException;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.Iterator;

public class FTPUtil
{
	
	/**
	 * Parses supplied user information which may be either name:password or
	 * just name
	 */
	public static Credentials getCredentials(String userInformation)
	{
		System.out.println(userInformation);
		if (userInformation == null)
		{
			return new Credentials("anonymous", "netkernel@1060research.com");
		}
		String[] result = userInformation.split(":");
		if (result.length > 1)
		{
			return new Credentials(result[0], result[1]);
		}
		else
		{
			return new Credentials(userInformation);
		}
	}

	public static void disconnectFromFTPServer(FileTransferClient ftpClient) throws NetKernelException
	{
		try
		{
			ftpClient.disconnect();
		}
		catch (Exception e)
		{
			throw new NetKernelException("Error occured while disconnecting from FTP server",
					"An error occured while disconnection from host [" + ftpClient.getRemoteHost() + "] on port ["
							+ ftpClient.getRemotePort() + "]");
		}
	}

	
	
	/** Strip path of ftp: uri to give a connection unique string
	 * @param aFTPURIString
	 * @return
	 */
	public static String toConnectionURIString(URI aFTPURI) throws URISyntaxException
	{	URI uri=new URI(aFTPURI.getScheme(),aFTPURI.getUserInfo(),aFTPURI.getHost(),aFTPURI.getPort(),"/",null,null);
		return uri.toString();
	}

	public static String getPath(URI aFTPURI) throws URISyntaxException
	{	return aFTPURI.getPath();
	}
	
	public static String changeDirAndGetFilename(FTPConnectionRepresentation aConnection, String aPath) throws FTPException, IOException
	{	FileTransferClient client=aConnection.getClient();
		String cwd=client.getRemoteDirectory();
		PathAndFile paf = new PathAndFile(aPath);
		//System.out.println("cwd="+cwd+" paf="+paf.getDirectory());
		if (!paf.getDirectory().equals(cwd))
		{	if (!cwd.equals("/"))
			{	int c=cwd.split("/").length;
				for (int i=0; i<c; i++)
				{	client.changeToParentDirectory();
				}
			}
			
			//client.changeToParentDirectory();
			Iterator iter = paf.getDirectoryPath();
			while (iter.hasNext())
			{	String directory = (String) iter.next();
				client.changeDirectory(directory);
			}
			
			
		}
			

		// Set transfer mode
		if (paf.isBinary())
		{	client.setContentType(FTPTransferType.BINARY);
		}
		else
		{	client.setContentType(FTPTransferType.ASCII);
		}
		return paf.getFileName();
	}



}
